% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/6_deprecated.R
\name{get_markers}
\alias{get_markers}
\title{get_markers}
\usage{
get_markers(ff, markers)
}
\arguments{
\item{ff}{The flowFrame of interest}

\item{markers}{Vector with markers or channels of interest}
}
\value{
Corresponding marker names
}
\description{
Get marker names, given a flowFrame. As available in "desc". If this is NA,
defaults to channel name.
}
\examples{

   # Read the flowFrame
   fileName <- system.file("extdata", "68983.fcs", package="FlowSOM")
   ff <- flowCore::read.FCS(fileName)
   GetChannels(ff, c("FSC-A", "CD3", "FITC-A"))
   GetMarkers(ff, c("FSC-A", "CD3", "FITC-A"))

}
\seealso{
\code{\link{get_channels}}
}
